@@ -140,6 +140,7 @@ createProjectTest pkgIx srcDb = testGroup "createProject tests"
               -- package name
               , "test-package"
               , "test-package"
+              , "test-package"
               -- version
               , "3.1.2.3"
               -- license
@@ -246,6 +247,7 @@ createProjectTest pkgIx srcDb = testGroup "createProject tests"
               -- package name
               , "test-package"
               , "test-package"
+              , "test-package"
               -- version
               , "3.1.2.3"
               -- license
@@ -322,7 +324,7 @@ createProjectTest pkgIx srcDb = testGroup "createProject tests"
             exe  @?= Nothing
             test @?! Nothing
           Left e -> assertFailure $ show e
-    
+
     , testCase "Check the interactive library workflow" $ do
         let inputs = fromList
               -- package type
@@ -337,6 +339,7 @@ createProjectTest pkgIx srcDb = testGroup "createProject tests"
               -- package name
               , "test-package"
               , "test-package"
+              , "test-package"
               -- version
               , "3.1.2.3"
               -- license
@@ -415,6 +418,7 @@ createProjectTest pkgIx srcDb = testGroup "createProject tests"
               -- package name
               , "test-package"
               , "test-package"
+              , "test-package"
               -- version
               , "3.1.2.3"
               -- license
@@ -507,6 +511,7 @@ createProjectTest pkgIx srcDb = testGroup "createProject tests"
               -- package name
               , "test-package"
               , "test-package"
+              , "test-package"
               -- version
               , "3.1.2.3"
               -- license
@@ -584,6 +589,7 @@ createProjectTest pkgIx srcDb = testGroup "createProject tests"
               -- package name
               , "test-package"
               , "test-package"
+              , "test-package"
               -- version
               , "3.1.2.3"
               -- license
@@ -667,6 +673,7 @@ createProjectTest pkgIx srcDb = testGroup "createProject tests"
               -- package name
               , "test-package"
               , "test-package"
+              , "test-package"
               -- version
               , "3.1.2.3"
               -- license
@@ -739,6 +746,7 @@ fileCreatorTests pkgIx srcDb _pkgName = testGroup "generators"
         let inputs = fromList
               [ "1"               -- pick the first cabal version in the list
               , "my-test-package" -- package name
+              , "my-test-package" -- current dir for the purpose of guessing the package name
               , "y"               -- "yes to prompt internal to package name"
               , "0.2.0.1"         -- package version
               , "2"               -- pick the second license in the list
@@ -796,15 +804,18 @@ interactiveTests srcDb = testGroup "Check top level getter functions"
           (packageNamePrompt srcDb) (mkPackageName "test-package")
           [ "test-package"
           , "test-package"
+          , "test-package"
           ]
       , testSimplePrompt "New package name 2"
           (packageNamePrompt srcDb) (mkPackageName "test-package")
           [ "test-package"
+          , "test-package"
           , ""
           ]
       , testSimplePrompt "Existing package name 1"
           (packageNamePrompt srcDb) (mkPackageName "test-package")
           [ "test-package"
+          , "test-package"
           , "cabal-install"
           , "y"
           , "test-package"
@@ -812,6 +823,7 @@ interactiveTests srcDb = testGroup "Check top level getter functions"
       , testSimplePrompt "Existing package name 2"
           (packageNamePrompt srcDb) (mkPackageName "cabal-install")
           [ "test-package"
+          , "test-package"
           , "cabal-install"
           , "n"
           ]
@@ -919,14 +931,13 @@ interactiveTests srcDb = testGroup "Check top level getter functions"
             , "Haskell2022"
             ]
         , testSimplePrompt "Invalid language"
-            (`languagePrompt` "test") Haskell2010
+            (`languagePrompt` "test") (UnknownLanguage "Lang_TS!")
             [ "4"
             , "Lang_TS!"
-            , "1"
             ]
         ]
     , testGroup "Check srcDirsPrompt output"
-        [ testNumberedPrompt "Soruce dirs indices" srcDirsPrompt
+        [ testNumberedPrompt "Source dirs indices" srcDirsPrompt
             [[defaultSourceDir], ["lib"], ["src-lib"]]
         , testSimplePrompt "Other source dir"
             srcDirsPrompt ["src"]
