@@ -9,23 +9,22 @@ import Prelude ()
 import Distribution.Client.Compat.Prelude
 
 import Control.Arrow ((&&&))
-import Control.DeepSeq (force)
 import Data.Either (lefts)
-import Data.Function (on)
 import Data.Hashable (Hashable(..))
 import Data.List (groupBy, isInfixOf)
-import Data.Ord (comparing)
 
 import Text.Show.Pretty (parseValue, valToStr)
 
 import Test.Tasty (TestTree)
-import Test.Tasty.QuickCheck
+import Test.QuickCheck (Arbitrary (..), Gen, Positive (..), frequency, oneof, shrinkList, shuffle, listOf, shrinkNothing, vectorOf, elements, sublistOf, counterexample, (===), (==>), Blind (..))
+import Test.QuickCheck.Instances.Cabal ()
 
-import Distribution.Types.GenericPackageDescription (FlagName)
+import Distribution.Types.Flag (FlagName)
 import Distribution.Utils.ShortText (ShortText)
 
 import Distribution.Client.Setup (defaultMaxBackjumps)
 
+import           Distribution.Types.LibraryVisibility
 import           Distribution.Types.PackageName
 import           Distribution.Types.UnqualComponentName
 
@@ -39,6 +38,7 @@ import           Distribution.Solver.Types.PkgConfigDb
                    (pkgConfigDbFromList)
 import           Distribution.Solver.Types.Settings
 import           Distribution.Solver.Types.Variable
+import           Distribution.Verbosity
 import           Distribution.Version
 
 import UnitTests.Distribution.Solver.Modular.DSL
@@ -56,8 +56,8 @@ tests = [
             let r1 = solve' mGoalOrder1 test
                 r2 = solve' mGoalOrder2 test { testTargets = targets2 }
                 solve' goalOrder =
-                    solve (EnableBackjumping True) (ReorderGoals False)
-                          (CountConflicts True) indepGoals
+                    solve (EnableBackjumping True) (FineGrainedConflicts True)
+                          (ReorderGoals False) (CountConflicts True) indepGoals (PreferOldest False)
                           (getBlind <$> goalOrder)
                 targets = testTargets test
                 targets2 = case targetOrder of
@@ -72,8 +72,9 @@ tests = [
           \test reorderGoals ->
             let r1 = solve' (IndependentGoals False) test
                 r2 = solve' (IndependentGoals True)  test
-                solve' indep = solve (EnableBackjumping True) reorderGoals
-                                     (CountConflicts True) indep Nothing
+                solve' indep =
+                    solve (EnableBackjumping True) (FineGrainedConflicts True)
+                          reorderGoals (CountConflicts True) indep (PreferOldest False)Nothing
              in counterexample (showResults r1 r2) $
                 noneReachedBackjumpLimit [r1, r2] ==>
                 isRight (resultPlan r1) `implies` isRight (resultPlan r2)
@@ -82,26 +83,63 @@ tests = [
           \test reorderGoals indepGoals ->
             let r1 = solve' (EnableBackjumping True)  test
                 r2 = solve' (EnableBackjumping False) test
-                solve' enableBj = solve enableBj reorderGoals
-                                        (CountConflicts True) indepGoals Nothing
+                solve' enableBj =
+                    solve enableBj (FineGrainedConflicts False) reorderGoals
+                          (CountConflicts True) indepGoals (PreferOldest False) Nothing
              in counterexample (showResults r1 r2) $
                 noneReachedBackjumpLimit [r1, r2] ==>
                 isRight (resultPlan r1) === isRight (resultPlan r2)
 
-    -- This test uses --no-count-conflicts, because the goal order used with
-    -- --count-conflicts depends on the total set of conflicts seen by the
+    , testPropertyWithSeed "fine-grained conflicts does not affect solvability" $
+          \test reorderGoals indepGoals ->
+            let r1 = solve' (FineGrainedConflicts True)  test
+                r2 = solve' (FineGrainedConflicts False) test
+                solve' fineGrainedConflicts =
+                    solve (EnableBackjumping True) fineGrainedConflicts
+                    reorderGoals (CountConflicts True) indepGoals (PreferOldest False) Nothing
+             in counterexample (showResults r1 r2) $
+                noneReachedBackjumpLimit [r1, r2] ==>
+                isRight (resultPlan r1) === isRight (resultPlan r2)
+
+    , testPropertyWithSeed "prefer oldest does not affect solvability" $
+          \test reorderGoals indepGoals ->
+            let r1 = solve' (PreferOldest True)  test
+                r2 = solve' (PreferOldest False) test
+                solve' prefOldest  =
+                    solve (EnableBackjumping True) (FineGrainedConflicts True)
+                    reorderGoals (CountConflicts True) indepGoals prefOldest Nothing
+             in counterexample (showResults r1 r2) $
+                noneReachedBackjumpLimit [r1, r2] ==>
+                isRight (resultPlan r1) === isRight (resultPlan r2)
+
+    -- The next two tests use --no-count-conflicts, because the goal order used
+    -- with --count-conflicts depends on the total set of conflicts seen by the
     -- solver. The solver explores more of the tree and encounters more
     -- conflicts when it doesn't backjump. The different goal orders can lead to
     -- different solutions and cause the test to fail.
     -- TODO: Find a faster way to randomly sort goals, and then use a random
-    -- goal order in this test.
+    -- goal order in these tests.
+
     , testPropertyWithSeed
           "backjumping does not affect the result (with static goal order)" $
           \test reorderGoals indepGoals ->
             let r1 = solve' (EnableBackjumping True)  test
                 r2 = solve' (EnableBackjumping False) test
-                solve' enableBj = solve enableBj reorderGoals
-                                  (CountConflicts False) indepGoals Nothing
+                solve' enableBj =
+                    solve enableBj (FineGrainedConflicts False) reorderGoals
+                          (CountConflicts False) indepGoals (PreferOldest False) Nothing
+             in counterexample (showResults r1 r2) $
+                noneReachedBackjumpLimit [r1, r2] ==>
+                resultPlan r1 === resultPlan r2
+
+    , testPropertyWithSeed
+          "fine-grained conflicts does not affect the result (with static goal order)" $
+          \test reorderGoals indepGoals ->
+            let r1 = solve' (FineGrainedConflicts True)  test
+                r2 = solve' (FineGrainedConflicts False) test
+                solve' fineGrainedConflicts =
+                    solve (EnableBackjumping True) fineGrainedConflicts
+                          reorderGoals (CountConflicts False) indepGoals (PreferOldest False) Nothing
              in counterexample (showResults r1 r2) $
                 noneReachedBackjumpLimit [r1, r2] ==>
                 resultPlan r1 === resultPlan r2
@@ -131,10 +169,16 @@ newtype VarOrdering = VarOrdering {
       unVarOrdering :: Variable P.QPN -> Variable P.QPN -> Ordering
     }
 
-solve :: EnableBackjumping -> ReorderGoals -> CountConflicts -> IndependentGoals
+solve :: EnableBackjumping
+      -> FineGrainedConflicts
+      -> ReorderGoals
+      -> CountConflicts
+      -> IndependentGoals
+      -> PreferOldest
       -> Maybe VarOrdering
-      -> SolverTest -> Result
-solve enableBj reorder countConflicts indep goalOrder test =
+      -> SolverTest
+      -> Result
+solve enableBj fineGrainedConflicts reorder countConflicts indep prefOldest goalOrder test =
   let (lg, result) =
         runProgress $ exResolve (unTestDb (testDb test)) Nothing Nothing
                   (pkgConfigDbFromList [])
@@ -142,9 +186,11 @@ solve enableBj reorder countConflicts indep goalOrder test =
                   -- The backjump limit prevents individual tests from using
                   -- too much time and memory.
                   (Just defaultMaxBackjumps)
-                  countConflicts indep reorder (AllowBootLibInstalls False)
-                  enableBj (SolveExecutables True) (unVarOrdering <$> goalOrder)
-                  (testConstraints test) (testPreferences test)
+                  countConflicts fineGrainedConflicts
+                  (MinimizeConflictSet False) indep prefOldest reorder
+                  (AllowBootLibInstalls False) OnlyConstrainedNone enableBj
+                  (SolveExecutables True) (unVarOrdering <$> goalOrder)
+                  (testConstraints test) (testPreferences test) normal
                   (EnableAllTests False)
 
       failure :: String -> Failure
@@ -266,7 +312,7 @@ instance Arbitrary TestDb where
 
 arbitraryExAv :: PN -> PV -> TestDb -> Gen ExampleAvailable
 arbitraryExAv pn v db =
-    (\cds -> ExAv (unPN pn) (unPV v) cds []) <$> arbitraryComponentDeps db
+    (\cds -> ExAv (unPN pn) (unPV v) cds []) <$> arbitraryComponentDeps pn db
 
 arbitraryExInst :: PN -> PV -> [ExampleInstalled] -> Gen ExampleInstalled
 arbitraryExInst pn v pkgs = do
@@ -275,15 +321,23 @@ arbitraryExInst pn v pkgs = do
   deps <- randomSubset numDeps pkgs
   return $ ExInst (unPN pn) (unPV v) pkgHash (map exInstHash deps)
 
-arbitraryComponentDeps :: TestDb -> Gen (ComponentDeps [ExampleDependency])
-arbitraryComponentDeps (TestDb []) = return $ CD.fromList []
-arbitraryComponentDeps db =
-    -- dedupComponentNames removes components with duplicate names, for example,
-    -- 'ComponentExe x' and 'ComponentTest x', and then CD.fromList combines
-    -- duplicate unnamed components.
-    CD.fromList . dedupComponentNames <$>
-    boundedListOf 5 (arbitraryComponentDep db)
+arbitraryComponentDeps :: PN -> TestDb -> Gen (ComponentDeps Dependencies)
+arbitraryComponentDeps _  (TestDb []) = return $ CD.fromLibraryDeps (dependencies [])
+arbitraryComponentDeps pn db          = do
+  -- dedupComponentNames removes components with duplicate names, for example,
+  -- 'ComponentExe x' and 'ComponentTest x', and then CD.fromList combines
+  -- duplicate unnamed components.
+  cds <- CD.fromList . dedupComponentNames . filter (isValid . fst)
+           <$> boundedListOf 5 (arbitraryComponentDep db)
+  return $ if isCompleteComponentDeps cds
+           then cds
+           else -- Add a library if the ComponentDeps isn't complete.
+                CD.fromLibraryDeps (dependencies []) <> cds
   where
+    isValid :: Component -> Bool
+    isValid (ComponentSubLib name) = name /= mkUnqualComponentName (unPN pn)
+    isValid _                      = True
+
     dedupComponentNames =
         nubBy ((\x y -> isJust x && isJust y && x == y) `on` componentName . fst)
 
@@ -296,13 +350,33 @@ arbitraryComponentDeps db =
     componentName (ComponentTest   n) = Just n
     componentName (ComponentBench  n) = Just n
 
-arbitraryComponentDep :: TestDb -> Gen (ComponentDep [ExampleDependency])
+-- | Returns true if the ComponentDeps forms a complete package, i.e., it
+-- contains a library, exe, test, or benchmark.
+isCompleteComponentDeps :: ComponentDeps a -> Bool
+isCompleteComponentDeps = any (completesPkg . fst) . CD.toList
+  where
+    completesPkg ComponentLib        = True
+    completesPkg (ComponentExe    _) = True
+    completesPkg (ComponentTest   _) = True
+    completesPkg (ComponentBench  _) = True
+    completesPkg (ComponentSubLib _) = False
+    completesPkg (ComponentFLib   _) = False
+    completesPkg ComponentSetup      = False
+
+arbitraryComponentDep :: TestDb -> Gen (ComponentDep Dependencies)
 arbitraryComponentDep db = do
   comp <- arbitrary
   deps <- case comp of
             ComponentSetup -> smallListOf (arbitraryExDep db SetupDep)
             _              -> boundedListOf 5 (arbitraryExDep db NonSetupDep)
-  return (comp, deps)
+  return ( comp
+         , Dependencies {
+             depsExampleDependencies = deps
+
+           -- TODO: Test different values for visibility and buildability.
+           , depsVisibility = LibraryVisibilityPublic
+           , depsIsBuildable = True
+           } )
 
 -- | Location of an 'ExampleDependency'. It determines which values are valid.
 data ExDepLocation = SetupDep | NonSetupDep
@@ -331,8 +405,8 @@ arbitraryExDep db@(TestDb pkgs) level =
 
 arbitraryDeps :: TestDb -> Gen Dependencies
 arbitraryDeps db = frequency
-    [ (1, return NotBuildable)
-    , (20, Buildable <$> smallListOf (arbitraryExDep db NonSetupDep))
+    [ (1, return unbuildableDependencies)
+    , (20, dependencies <$> smallListOf (arbitraryExDep db NonSetupDep))
     ]
 
 arbitraryFlagName :: Gen String
@@ -376,22 +450,27 @@ instance Arbitrary IndependentGoals where
 
   shrink (IndependentGoals indep) = [IndependentGoals False | indep]
 
-instance Arbitrary UnqualComponentName where
-  arbitrary = mkUnqualComponentName <$> (:[]) <$> elements "ABC"
-
 instance Arbitrary Component where
   arbitrary = oneof [ return ComponentLib
-                    , ComponentSubLib <$> arbitrary
-                    , ComponentExe <$> arbitrary
-                    , ComponentFLib <$> arbitrary
-                    , ComponentTest <$> arbitrary
-                    , ComponentBench <$> arbitrary
+                    , ComponentSubLib <$> arbitraryUQN
+                    , ComponentExe <$> arbitraryUQN
+                    , ComponentFLib <$> arbitraryUQN
+                    , ComponentTest <$> arbitraryUQN
+                    , ComponentBench <$> arbitraryUQN
                     , return ComponentSetup
                     ]
 
   shrink ComponentLib = []
   shrink _ = [ComponentLib]
 
+-- The "component-" prefix prevents component names and build-depends
+-- dependency names from overlapping.
+-- TODO: Remove the prefix once the QuickCheck tests support dependencies on
+-- internal libraries.
+arbitraryUQN :: Gen UnqualComponentName
+arbitraryUQN =
+    mkUnqualComponentName <$> (\c -> "component-" ++ [c]) <$> elements "ABC"
+
 instance Arbitrary ExampleInstalled where
   arbitrary = error "arbitrary not implemented: ExampleInstalled"
 
@@ -406,7 +485,7 @@ instance Arbitrary ExampleAvailable where
 instance (Arbitrary a, Monoid a) => Arbitrary (ComponentDeps a) where
   arbitrary = error "arbitrary not implemented: ComponentDeps"
 
-  shrink = map CD.fromList . shrink . CD.toList
+  shrink = filter isCompleteComponentDeps . map CD.fromList . shrink . CD.toList
 
 instance Arbitrary ExampleDependency where
   arbitrary = error "arbitrary not implemented: ExampleDependency"
@@ -415,19 +494,18 @@ instance Arbitrary ExampleDependency where
   shrink (ExFix "base" _) = [] -- preserve bounds on base
   shrink (ExFix pn _) = [ExAny pn]
   shrink (ExFlagged flag th el) =
-         deps th ++ deps el
+         depsExampleDependencies th ++ depsExampleDependencies el
       ++ [ExFlagged flag th' el | th' <- shrink th]
       ++ [ExFlagged flag th el' | el' <- shrink el]
-    where
-      deps NotBuildable = []
-      deps (Buildable ds) = ds
   shrink dep = error $ "Dependency not handled: " ++ show dep
 
 instance Arbitrary Dependencies where
   arbitrary = error "arbitrary not implemented: Dependencies"
 
-  shrink NotBuildable = [Buildable []]
-  shrink (Buildable deps) = map Buildable (shrink deps)
+  shrink deps =
+         [ deps { depsVisibility = v } | v <- shrink $ depsVisibility deps ]
+      ++ [ deps { depsIsBuildable = b } | b <- shrink $ depsIsBuildable deps ]
+      ++ [ deps { depsExampleDependencies = ds } | ds <- shrink $ depsExampleDependencies deps ]
 
 instance Arbitrary ExConstraint where
   arbitrary = error "arbitrary not implemented: ExConstraint"
@@ -451,11 +529,6 @@ instance Arbitrary OptionalStanza where
   shrink BenchStanzas = [TestStanzas]
   shrink TestStanzas  = []
 
-instance Arbitrary VersionRange where
-  arbitrary = error "arbitrary not implemented: VersionRange"
-
-  shrink vr = [noVersion | vr /= noVersion]
-
 -- Randomly sorts solver variables using 'hash'.
 -- TODO: Sorting goals with this function is very slow.
 instance Arbitrary VarOrdering where
